@@ -31,6 +31,10 @@ def _access_through_cftimeindex(values, name):
     if name == "season":
         months = values_as_cftimeindex.month
         field_values = _season_from_months(months)
+    elif name == "date":
+        raise AttributeError(
+            "'CFTimeIndex' object has no attribute `date`. Consider using the floor method instead, for instance: `.time.dt.floor('D')`."
+        )
     else:
         field_values = getattr(values_as_cftimeindex, name)
     return field_values.reshape(values.shape)
@@ -259,8 +263,6 @@ class DatetimeAccessor(Properties):
 
     Examples
     ---------
-    >>> import xarray as xr
-    >>> import pandas as pd
     >>> dates = pd.date_range(start="2000/01/01", freq="D", periods=10)
     >>> ts = xr.DataArray(dates, dims=("time"))
     >>> ts
@@ -415,6 +417,10 @@ class DatetimeAccessor(Properties):
         "time", "Timestamps corresponding to datetimes", object
     )
 
+    date = Properties._tslib_field_accessor(
+        "date", "Date corresponding to datetimes", object
+    )
+
     is_month_start = Properties._tslib_field_accessor(
         "is_month_start",
         "Indicates whether the date is the first day of the month.",
@@ -451,8 +457,6 @@ class TimedeltaAccessor(Properties):
 
     Examples
     --------
-    >>> import pandas as pd
-    >>> import xarray as xr
     >>> dates = pd.timedelta_range(start="1 day", freq="6H", periods=20)
     >>> ts = xr.DataArray(dates, dims=("time"))
     >>> ts
