@@ -3,9 +3,18 @@ import numbers
 
 import numpy as np
 
+# _typed_ops.py is a generated file
+from ._typed_ops import (
+    DataArrayGroupByOpsMixin,
+    DataArrayOpsMixin,
+    DatasetGroupByOpsMixin,
+    DatasetOpsMixin,
+    VariableOpsMixin,
+)
+from .common import ImplementsArrayReduce, ImplementsDatasetReduce
+from .ops import IncludeCumMethods, IncludeNumpySameMethods, IncludeReduceMethods
 from .options import OPTIONS, _get_keep_attrs
 from .pycompat import dask_array_type
-from .utils import not_implemented
 
 
 class SupportsArithmetic:
@@ -80,26 +89,61 @@ class SupportsArithmetic:
             keep_attrs=_get_keep_attrs(default=True),
         )
 
-    # this has no runtime function - these are listed so IDEs know these
-    # methods are defined and don't warn on these operations
-    __lt__ = (
-        __le__
-    ) = (
-        __ge__
-    ) = (
-        __gt__
-    ) = (
-        __add__
-    ) = (
-        __sub__
-    ) = (
-        __mul__
-    ) = (
-        __truediv__
-    ) = (
-        __floordiv__
-    ) = (
-        __mod__
-    ) = (
-        __pow__
-    ) = __and__ = __xor__ = __or__ = __div__ = __eq__ = __ne__ = not_implemented
+
+class VariableArithmetic(
+    ImplementsArrayReduce,
+    IncludeReduceMethods,
+    IncludeCumMethods,
+    IncludeNumpySameMethods,
+    SupportsArithmetic,
+    VariableOpsMixin,
+):
+    __slots__ = ()
+    # prioritize our operations over those of numpy.ndarray (priority=0)
+    __array_priority__ = 50
+
+
+class DatasetArithmetic(
+    ImplementsDatasetReduce,
+    IncludeReduceMethods,
+    IncludeCumMethods,
+    SupportsArithmetic,
+    DatasetOpsMixin,
+):
+    __slots__ = ()
+    __array_priority__ = 50
+
+
+class DataArrayArithmetic(
+    ImplementsArrayReduce,
+    IncludeReduceMethods,
+    IncludeCumMethods,
+    IncludeNumpySameMethods,
+    SupportsArithmetic,
+    DataArrayOpsMixin,
+):
+    __slots__ = ()
+    # priority must be higher than Variable to properly work with binary ufuncs
+    __array_priority__ = 60
+
+
+class DataArrayGroupbyArithmetic(
+    ImplementsArrayReduce,
+    IncludeReduceMethods,
+    SupportsArithmetic,
+    DataArrayGroupByOpsMixin,
+):
+    __slots__ = ()
+
+
+class DatasetGroupbyArithmetic(
+    ImplementsDatasetReduce,
+    IncludeReduceMethods,
+    SupportsArithmetic,
+    DatasetGroupByOpsMixin,
+):
+    __slots__ = ()
+
+
+class CoarsenArithmetic(IncludeReduceMethods):
+    __slots__ = ()
