@@ -143,8 +143,9 @@ def concat(
           those of the first object with that dimension. Indexes for the same
           dimension must have the same size in all objects.
     combine_attrs : {"drop", "identical", "no_conflicts", "drop_conflicts", \
-                     "override"}, default: "override"
-        String indicating how to combine attrs of the objects being merged:
+                     "override"} or callable, default: "override"
+        A callable or a string indicating how to combine attrs of the objects being
+        merged:
 
         - "drop": empty attrs on returned Dataset.
         - "identical": all attrs must be the same on every object.
@@ -155,6 +156,9 @@ def concat(
         - "override": skip comparing and copy attrs from the first dataset to
           the result.
 
+        If a callable, it must expect a sequence of ``attrs`` dicts and a context object
+        as its only parameters.
+
     Returns
     -------
     concatenated : type of objs
@@ -223,8 +227,7 @@ def concat(
 
     if compat not in _VALID_COMPAT:
         raise ValueError(
-            "compat=%r invalid: must be 'broadcast_equals', 'equals', 'identical', 'no_conflicts' or 'override'"
-            % compat
+            f"compat={compat!r} invalid: must be 'broadcast_equals', 'equals', 'identical', 'no_conflicts' or 'override'"
         )
 
     if isinstance(first_obj, DataArray):
@@ -234,7 +237,7 @@ def concat(
     else:
         raise TypeError(
             "can only concatenate xarray Dataset and DataArray "
-            "objects, got %s" % type(first_obj)
+            f"objects, got {type(first_obj)}"
         )
     return f(
         objs, dim, data_vars, coords, compat, positions, fill_value, join, combine_attrs
@@ -293,18 +296,16 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars, coords, compat):
             if opt == "different":
                 if compat == "override":
                     raise ValueError(
-                        "Cannot specify both %s='different' and compat='override'."
-                        % subset
+                        f"Cannot specify both {subset}='different' and compat='override'."
                     )
                 # all nonindexes that are not the same in each dataset
                 for k in getattr(datasets[0], subset):
                     if k not in concat_over:
                         equals[k] = None
 
-                        variables = []
-                        for ds in datasets:
-                            if k in ds.variables:
-                                variables.append(ds.variables[k])
+                        variables = [
+                            ds.variables[k] for ds in datasets if k in ds.variables
+                        ]
 
                         if len(variables) == 1:
                             # coords="different" doesn't make sense when only one object
@@ -367,12 +368,12 @@ def _calc_concat_over(datasets, dim, dim_names, data_vars, coords, compat):
                 if subset == "coords":
                     raise ValueError(
                         "some variables in coords are not coordinates on "
-                        "the first dataset: %s" % (invalid_vars,)
+                        f"the first dataset: {invalid_vars}"
                     )
                 else:
                     raise ValueError(
                         "some variables in data_vars are not data variables "
-                        "on the first dataset: %s" % (invalid_vars,)
+                        f"on the first dataset: {invalid_vars}"
                     )
             concat_over.update(opt)
 
@@ -425,6 +426,13 @@ def _dataset_concat(
     """
     from .dataset import Dataset
 
+    datasets = list(datasets)
+
+    if not all(isinstance(dataset, Dataset) for dataset in datasets):
+        raise TypeError(
+            "The elements in the input list need to be either all 'Dataset's or all 'DataArray's"
+        )
+
     dim, coord = _calc_concat_dim_coord(dim)
     # Make sure we're working on a copy (we'll be loading variables)
     datasets = [ds.copy() for ds in datasets]
@@ -439,7 +447,7 @@ def _dataset_concat(
     both_data_and_coords = coord_names & data_names
     if both_data_and_coords:
         raise ValueError(
-            "%r is a coordinate in some datasets but not others." % both_data_and_coords
+            f"{both_data_and_coords!r} is a coordinate in some datasets but not others."
         )
     # we don't want the concat dimension in the result dataset yet
     dim_coords.pop(dim, None)
@@ -507,8 +515,8 @@ def _dataset_concat(
             try:
                 vars = ensure_common_dims([ds[k].variable for ds in datasets])
             except KeyError:
-                raise ValueError("%r is not present in all datasets." % k)
-            combined = concat_vars(vars, dim, positions)
+                raise ValueError(f"{k!r} is not present in all datasets.")
+            combined = concat_vars(vars, dim, positions, combine_attrs=combine_attrs)
             assert isinstance(combined, Variable)
             result_vars[k] = combined
         elif k in result_vars:
@@ -519,8 +527,7 @@ def _dataset_concat(
     absent_coord_names = coord_names - set(result.variables)
     if absent_coord_names:
         raise ValueError(
-            "Variables %r are coordinates in some datasets but not others."
-            % absent_coord_names
+            f"Variables {absent_coord_names!r} are coordinates in some datasets but not others."
         )
     result = result.set_coords(coord_names)
     result.encoding = result_encoding
@@ -545,8 +552,15 @@ def _dataarray_concat(
     join: str = "outer",
     combine_attrs: str = "override",
 ) -> "DataArray":
+    from .dataarray import DataArray
+
     arrays = list(arrays)
 
+    if not all(isinstance(array, DataArray) for array in arrays):
+        raise TypeError(
+            "The elements in the input list need to be either all 'Dataset's or all 'DataArray's"
+        )
+
     if data_vars != "all":
         raise ValueError(
             "data_vars is not a valid argument when concatenating DataArray objects"
@@ -572,7 +586,7 @@ def _dataarray_concat(
         positions,
         fill_value=fill_value,
         join=join,
-        combine_attrs="drop",
+        combine_attrs=combine_attrs,
     )
 
     merged_attrs = merge_attrs([da.attrs for da in arrays], combine_attrs)
