@@ -11,9 +11,9 @@ import pandas as pd
 from . import utils
 from .common import _contains_datetime_like_objects, ones_like
 from .computation import apply_ufunc
-from .duck_array_ops import datetime_to_numeric, timedelta_to_numeric
+from .duck_array_ops import datetime_to_numeric, push, timedelta_to_numeric
 from .options import _get_keep_attrs
-from .pycompat import is_duck_dask_array
+from .pycompat import dask_version, is_duck_dask_array
 from .utils import OrderedSet, is_scalar
 from .variable import Variable, broadcast_variables
 
@@ -93,7 +93,7 @@ class NumpyInterpolator(BaseInterpolator):
             self._left = fill_value
             self._right = fill_value
         else:
-            raise ValueError("%s is not a valid fill_value" % fill_value)
+            raise ValueError(f"{fill_value} is not a valid fill_value")
 
     def __call__(self, x):
         return self.f(
@@ -317,9 +317,13 @@ def interp_na(
         if not is_scalar(max_gap):
             raise ValueError("max_gap must be a scalar.")
 
+        # TODO: benbovy - flexible indexes: update when CFTimeIndex (and DatetimeIndex?)
+        # has its own class inheriting from xarray.Index
         if (
-            dim in self.indexes
-            and isinstance(self.indexes[dim], (pd.DatetimeIndex, CFTimeIndex))
+            dim in self.xindexes
+            and isinstance(
+                self.xindexes[dim].to_pandas_index(), (pd.DatetimeIndex, CFTimeIndex)
+            )
             and use_coordinate
         ):
             # Convert to float
@@ -390,12 +394,10 @@ def func_interpolate_na(interpolator, y, x, **kwargs):
 
 def _bfill(arr, n=None, axis=-1):
     """inverse of ffill"""
-    import bottleneck as bn
-
     arr = np.flip(arr, axis=axis)
 
     # fill
-    arr = bn.push(arr, axis=axis, n=n)
+    arr = push(arr, axis=axis, n=n)
 
     # reverse back to original
     return np.flip(arr, axis=axis)
@@ -403,17 +405,15 @@ def _bfill(arr, n=None, axis=-1):
 
 def ffill(arr, dim=None, limit=None):
     """forward fill missing values"""
-    import bottleneck as bn
-
     axis = arr.get_axis_num(dim)
 
     # work around for bottleneck 178
     _limit = limit if limit is not None else arr.shape[axis]
 
     return apply_ufunc(
-        bn.push,
+        push,
         arr,
-        dask="parallelized",
+        dask="allowed",
         keep_attrs=True,
         output_dtypes=[arr.dtype],
         kwargs=dict(n=_limit, axis=axis),
@@ -430,7 +430,7 @@ def bfill(arr, dim=None, limit=None):
     return apply_ufunc(
         _bfill,
         arr,
-        dask="parallelized",
+        dask="allowed",
         keep_attrs=True,
         output_dtypes=[arr.dtype],
         kwargs=dict(n=_limit, axis=axis),
@@ -621,10 +621,6 @@ def interp(var, indexes_coords, method, **kwargs):
     for indexes_coords in decompose_interp(indexes_coords):
         var = result
 
-        # simple speed up for the local interpolation
-        if method in ["linear", "nearest"]:
-            var, indexes_coords = _localize(var, indexes_coords)
-
         # target dimensions
         dims = list(indexes_coords)
         x, new_x = zip(*[indexes_coords[d] for d in dims])
@@ -695,21 +691,22 @@ def interp_func(var, x, new_x, method, kwargs):
     if is_duck_dask_array(var):
         import dask.array as da
 
-        nconst = var.ndim - len(x)
+        ndim = var.ndim
+        nconst = ndim - len(x)
 
-        out_ind = list(range(nconst)) + list(range(var.ndim, var.ndim + new_x[0].ndim))
+        out_ind = list(range(nconst)) + list(range(ndim, ndim + new_x[0].ndim))
 
         # blockwise args format
         x_arginds = [[_x, (nconst + index,)] for index, _x in enumerate(x)]
         x_arginds = [item for pair in x_arginds for item in pair]
         new_x_arginds = [
-            [_x, [var.ndim + index for index in range(_x.ndim)]] for _x in new_x
+            [_x, [ndim + index for index in range(_x.ndim)]] for _x in new_x
         ]
         new_x_arginds = [item for pair in new_x_arginds for item in pair]
 
         args = (
             var,
-            range(var.ndim),
+            range(ndim),
             *x_arginds,
             *new_x_arginds,
         )
@@ -721,7 +718,7 @@ def interp_func(var, x, new_x, method, kwargs):
         new_x = rechunked[1 + (len(rechunked) - 1) // 2 :]
 
         new_axes = {
-            var.ndim + i: new_x[0].chunks[i]
+            ndim + i: new_x[0].chunks[i]
             if new_x[0].chunks is not None
             else new_x[0].shape[i]
             for i in range(new_x[0].ndim)
@@ -737,6 +734,13 @@ def interp_func(var, x, new_x, method, kwargs):
         else:
             dtype = var.dtype
 
+        if dask_version < "2020.12":
+            # Using meta and dtype at the same time doesn't work.
+            # Remove this whenever the minimum requirement for dask is 2020.12:
+            meta = None
+        else:
+            meta = var._meta
+
         return da.blockwise(
             _dask_aware_interpnd,
             out_ind,
@@ -747,6 +751,8 @@ def interp_func(var, x, new_x, method, kwargs):
             concatenate=True,
             dtype=dtype,
             new_axes=new_axes,
+            meta=meta,
+            align_arrays=False,
         )
 
     return _interpnd(var, x, new_x, func, kwargs)
