@@ -1,4 +1,5 @@
-from typing import TYPE_CHECKING, Generic, Hashable, Mapping, Optional, TypeVar
+from distutils.version import LooseVersion
+from typing import TYPE_CHECKING, Generic, Hashable, Mapping, TypeVar, Union
 
 import numpy as np
 
@@ -26,12 +27,25 @@ def move_exp_nanmean(array, *, axis, alpha):
         raise TypeError("rolling_exp is not currently support for dask-like arrays")
     import numbagg
 
+    # No longer needed in numbag > 0.2.0; remove in time
     if axis == ():
         return array.astype(np.float64)
     else:
         return numbagg.move_exp_nanmean(array, axis=axis, alpha=alpha)
 
 
+def move_exp_nansum(array, *, axis, alpha):
+    if is_duck_dask_array(array):
+        raise TypeError("rolling_exp is not currently supported for dask-like arrays")
+    import numbagg
+
+    # numbagg <= 0.2.0 did not have a __version__ attribute
+    if LooseVersion(getattr(numbagg, "__version__", "0.1.0")) < LooseVersion("0.2.0"):
+        raise ValueError("`rolling_exp(...).sum() requires numbagg>=0.2.1.")
+
+    return numbagg.move_exp_nansum(array, axis=axis, alpha=alpha)
+
+
 def _get_center_of_mass(comass, span, halflife, alpha):
     """
     Vendored from pandas.core.window.common._get_center_of_mass
@@ -74,7 +88,7 @@ class RollingExp(Generic[T_DSorDA]):
     ----------
     obj : Dataset or DataArray
         Object to window.
-    windows : mapping of hashable to int
+    windows : mapping of hashable to int (or float for alpha type)
         A mapping from the name of the dimension to create the rolling
         exponential window along (e.g. `time`) to the size of the moving window.
     window_type : {"span", "com", "halflife", "alpha"}, default: "span"
@@ -90,7 +104,7 @@ class RollingExp(Generic[T_DSorDA]):
     def __init__(
         self,
         obj: T_DSorDA,
-        windows: Mapping[Hashable, int],
+        windows: Mapping[Hashable, Union[int, float]],
         window_type: str = "span",
     ):
         self.obj: T_DSorDA = obj
@@ -98,9 +112,9 @@ class RollingExp(Generic[T_DSorDA]):
         self.dim = dim
         self.alpha = _get_alpha(**{window_type: window})
 
-    def mean(self, keep_attrs: Optional[bool] = None) -> T_DSorDA:
+    def mean(self, keep_attrs: bool = None) -> T_DSorDA:
         """
-        Exponentially weighted moving average
+        Exponentially weighted moving average.
 
         Parameters
         ----------
@@ -124,3 +138,30 @@ class RollingExp(Generic[T_DSorDA]):
         return self.obj.reduce(
             move_exp_nanmean, dim=self.dim, alpha=self.alpha, keep_attrs=keep_attrs
         )
+
+    def sum(self, keep_attrs: bool = None) -> T_DSorDA:
+        """
+        Exponentially weighted moving sum.
+
+        Parameters
+        ----------
+        keep_attrs : bool, default: None
+            If True, the attributes (``attrs``) will be copied from the original
+            object to the new one. If False, the new object will be returned
+            without attributes. If None uses the global default.
+
+        Examples
+        --------
+        >>> da = xr.DataArray([1, 1, 2, 2, 2], dims="x")
+        >>> da.rolling_exp(x=2, window_type="span").sum()
+        <xarray.DataArray (x: 5)>
+        array([1.        , 1.33333333, 2.44444444, 2.81481481, 2.9382716 ])
+        Dimensions without coordinates: x
+        """
+
+        if keep_attrs is None:
+            keep_attrs = _get_keep_attrs(default=True)
+
+        return self.obj.reduce(
+            move_exp_nansum, dim=self.dim, alpha=self.alpha, keep_attrs=keep_attrs
+        )
